#!/usr/bin/perl
use lib '/home/erealms/ethereal/mgmt/perl';

#################################################################################
# Created       : Martin Foster
# Modified      : 04-Jun-2006
#################################################################################
#
# Configure Puppeteer - Script part of Ethereal Realms, designed to configure
#                       a puppeteers settings
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 3456 Wolfe Cres
#                 Halifax, Nova Scotia
#                 B3L 3S2
#
#################################################################################

use CGI;							# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Digest::MD5 qw(md5_hex);                                    # MD5 digest cryptographic algorithm
use Lingua::EN::NameCase 'NameCase';				# Namecase handling
use strict;							# Strict variable enforcement

use Ethereal::Audit;						# Auditing handle
use Ethereal::Database;						# Database handler
use Ethereal::Filter;						# Filter handler
use Ethereal::Geo;						# Geographical handler
use Ethereal::Home;						# Homepage handler
use Ethereal::Image;						# Image handler
use Ethereal::Login;						# Login functionality
use Ethereal::Option;						# Option handlings
use Ethereal::Param;						# Parameter control
use Ethereal::Template;						# Template handler

#################################################################################
# Gobal override
#################################################################################
$CGI::POST_MAX=1024 * 2048;					# Maximum posts
$CGI::DISABLE_UPLOADS = 0;					# Disable uploads

#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter hash
my $tmpl;							# Template handle

my $pass;							# Passed on information
my $upload;							# Upload information

my %config;							# Configuration tag

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;	
	$database = new Ethereal::Database();

	# Connect and fetch
	$database->Connect($cgi);


	# Prefetch uploads
	$upload	= $cgi->upload('UPLOAD_VANITY');

	# Parameter handling
	$param    = new Ethereal::Param($database, $cgi);
	$param->GetParam();

	# Fetch hash
	$database->GetHashUser(\%config);

	# Template handle
	$tmpl = new Ethereal::Template(\%config);


	# Login handle
	$login    = new Ethereal::Login($database, $cgi, $param);

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		if ((defined($param->{'GEO'}))
		 && ($param->{'GEO'} eq 'geo'))
	 	{
			# Requires own header and footer
			# Keep segregated
			SocialLocate($database, $cgi, $param, $tmpl, \%config);
		}

		else
		{
			# Document header
			print $database->DocumentGetHeader(), "\n";


			# Chat preferences handling
			if (defined($param->{'SETPREFS'}))
			{ ChatHandler($database, $param, \%config); }
	
			# Information handling
			$pass = InformationHandler($database, $cgi, $param, \%config)
			 if (defined($param->{'OLDPASS'})
			  || defined($param->{'EMAIL'}));

			# Ensure existence
			$pass = (defined($pass)) ? $pass : '';

			# Initial screen	
			InitialDisplay($database, $cgi, $param, $tmpl, \%config, $pass, $upload);

			# Document footer
			print $database->DocumentGetFooter(), "\n";
		}
	}


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Chat Prefereces Handler
#
# Takes care of the database updates when one wants to change chat preferences
# this is done by figuring out what the options where.

sub ChatHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration tags	

	my $option;						# Options handle
	my $rjava;						# Formated javascript
	my $rchat;						# Formatted chat preferences
	my $rformat;						# Formatted format options
	my $rignore;						# Formatted ignore options
	my $rmailer;						# Formatted mailer behaviour
	my $rprivacy;						# Privacy Options


	#####################
	# Program area

	# Ensure definition
	$param->{'CIGNORE'} = 'yes' unless (defined($param->{'CIGNORE'}));

	# Create instance
	$option   = new Ethereal::Option();

	# Verify options received
	$rjava    = $option->FindJavascript($config->{'OptJavascript'}, $param->{'CJAVA'});
	$rchat    = $option->FindChatPrefs($config->{'OptChatPref'}, $param->{'CCHAT'});
	$rformat  = $option->FindFormatting($config->{'OptFormat'}, $param->{'CFORMAT'});
	$rignore  = $option->FindScope($config->{'OptScope'}, $param->{'CIGNORE'});
	$rmailer  = $option->FindMailer($config->{'OptMailer'}, $param->{'CMAILER'});
	$rprivacy = $option->FindPrivacy($config->{'OptPrivacy'}, $param->{'CPRIVACY'});


	# Evaluate
	eval
	{
		# Update database entry
		$database->Write("UPDATE Puppeteer
			SET   PuppeteerJavascript=?,
			      PuppeteerChatPref=?,
			      PuppeteerFormatting=?,
			      PuppeteerIgnore=?,
			      PuppeteerMailer=?,
			      PuppeteerPrivacy=?
			WHERE PuppeteerLogin=?",

		 $rjava,
		 $rchat,
		 $rformat,
		 $rignore,
		 $rmailer,
		 $rprivacy,
		 $param->{'USER'}
		);

		# Commit changes
		$database->Commit();
	};

	# If block failed
	if ($@)
	{
		# Issue warning
		warn("Transaction aborted: $@");

		# Undo incomplete changes
		$database->Rollback();
	}
}


#####################
# Check Data
#  
# Arbitrary sub-routine designed to be used to check validity of data.  Made
# to hide ugliness of checks also centralize adapting of checks.

sub CheckData
{
	#####################
	# Data members
	my $value    = shift;					# Value to check 
	my $choice   = shift;					# Choice of the matter
	my $database = shift;					# Possible database handle
	my $user     = shift;					# User


	#####################
	# Program area

	# Password Check
	if ($choice == 0)
	{
		# Retreive Password
		my ($crypt) = $database->DataGet("SELECT PuppeteerPassword
			FROM  Puppeteer
			WHERE PuppeteerLogin=?", $user);	


		# Verify existence of user and validity of password
		if ((defined($crypt)) && ($crypt eq crypt(substr($value, 0, 8), $crypt)))
		{ return 1; }					# Return true
	}

	# Length
	elsif ($choice == 1)
	{
		# Lenght of a minumum of 3 characters
		if (length($value) > 3)
		{ return 1; }					# Return true
	}

	# E-Mail
	elsif ($choice == 2)
	{
		# Verify for legality
		if ($value =~ /^\w(\w|\.|\-)*?\@(\w|\-)*?\..*?/s)
		{ return 1; }					# Return true
	}

	# HTTP
	elsif ($choice == 3)
	{
		# Verify for legality
		if ($value =~ /^http\:\/\//s)
		{ return 1; }					# Return true
	}

	# Numeric
	elsif ($choice == 4)
	{
		# Verify for legality
		if ($value =~ /^\d+$/s)
		{ return 1; }					# Return true
	}

	# If all else fails
	return 0; 
}

# Alias for Password validity
sub CheckDataPassword
{ CheckData($_[2], 0, $_[0], $_[1]); }

# Alias for Lenght
sub CheckDataLenght
{ CheckData(shift, 1); }

# Alias for E-Eail validity
sub CheckDataEMail
{ CheckData(shift, 2); }

# Alias for URL addresses
sub CheckDataAddress
{ CheckData(shift, 3); }

# Alias for Numeric filter
sub CheckDataNumeric
{ CheckData(shift, 4); }


#####################
# Contact Handler
#
# This sub-routine will allow the user to add in information about certain 
# contact information.    This is a necessary component for the enhaced gummy
# functionality.

sub ContactHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Configuration hash
	my $links    = shift;					# Hash of links

	my $res;						# Search results
	my $table;						# Table handler
	my $statement;						# Search statements

	my $code;						# Title code
	my $info;						# Parameter date
	my $name;						# Parameter name
	my $type;						# Contact type
	my $user;						# Username

	my $item;						# Template of items

	my %user;						# User results


	#####################
	# Program area	
	
	# Initial assignment
	$item = '';
	$type = 'None';
	$user = $param->{'USER'};


	# Initial user search and gather
	$database->Pull(\$statement, "SELECT
		  ContactName AS \"ContactName\", 
		  ContactData AS \"ContactData\"
		FROM PuppeteerContact
		WHERE PuppeteerLogin=?",

	 $param->{'USER'}
	);

	# Loop and gather
	while ($res = $statement->fetchrow_hashref())
	{
		# Store in hash for quick reference
		$user{$res->{'ContactName'}} = $res->{'ContactData'};
	}

	# End query
	$statement->finish();



	# Main operation
	# Prepare and execute
	$database->Pull(\$statement, "SELECT
		 ContactName   AS \"ContactName\",
		 ContactType   AS \"ContactType\",
		 ContactFilter AS \"ContactFilter\"
		FROM Contact
		ORDER BY ContactType, ContactName");


	# Loop and handle
	while ($res = $statement->fetchrow_hashref())
	{
		# Adjust name
		$name =  $res->{'ContactName'};
		$name =~ s/\W//gs;

		# Initial assigment
		$info =  $param->{$name};

		# Cleanup
		$param->Cleanup($name);


		# Ensure initial form displayed
		if (defined($param->{'PROCESS'}))
		{
			# Information check
		 	if (defined($info))	
			{
				# Email based search
				if ($res->{'ContactFilter'} eq 'email')
				{ $info = CheckDataEMail($info) ? $info : undef; }

				# Web address confirm
				elsif ($res->{'ContactFilter'} eq 'http')
				{ $info = CheckDataAddress($info) ? $info : undef; }

				# Numeric based acceptance
				elsif ($res->{'ContactFilter'} eq 'numeric')
				{ $info = CheckDataNumeric($info) ? $info : undef; }

				# Default filter
				else
				{ $info = CheckDataLenght($info) ? $info : undef; }
			}


			# Determine need
			# Creation
			if ((defined($info))
			 && (!defined($user{$res->{'ContactName'}})))
			{
				# Create parameter
				$database->Quick("INSERT INTO PuppeteerContact
					(ContactName,
					 ContactData,
					 PuppeteerLogin)
					VALUES(?,?,?)",

				 $res->{'ContactName'},
				 $info,
				 $user
				);
			}

			# Removal
			elsif ((!defined($info))
			 && (defined($user{$res->{'ContactName'}}))) 
			{
				# Remove parameter
				$database->Quick("DELETE FROM PuppeteerContact
					WHERE ContactName=?
					  AND PuppeteerLogin=?",

				 $res->{'ContactName'},
				 $user
				);
			}

			# Modify
			elsif ((defined($info))
			 && (defined($user{$res->{'ContactName'}}))
			 && ($info ne $user{$res->{'ContactName'}}))
			{
				# Modify parameter
				$database->Quick("UPDATE PuppeteerContact
					SET ContactData=?
					WHERE ContactName=?
					  AND PuppeteerLogin=?",

				 $info,
				 $res->{'ContactName'},
				 $user
				);
			}
		}

		# Initial pickup
		else
		{
			# Assign current value
			$info = $user{$res->{'ContactName'}};
		}

		# Final assignment
		$info = (defined($info)) ? $info : '';


		# Information display
		# Ensure defined
		$res->{'ContactType'} = (defined($res->{'ContactType'}))
		  ? $res->{'ContactType'}
		  : 'None';

		# Display section
		if (ucfirst($res->{'ContactType'}) ne $type)
		{
			# Massage
			$type =  ucfirst($res->{'ContactType'});

			# Append
			$item .= $tmpl->Pass('TmplUserContactType',
				MNAME => $type
			 );
		}

		# General append
		$item .= $tmpl->Pass('TmplUserContactItem',
			MNAME    => $res->{'ContactName'},
			WCONTROL => $cgi->textfield($name, $info, 35, 200)
		 );
	}

	# Information display
	# Form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), PROCESS=>'True'), "\n";

	# Display template
	$tmpl->Show('TmplUserContact',
		MLINKS  => $links,
		WRESET  => $cgi->submit($config->{'TxtReset'}),
		WSUBMIT => $cgi->submit($config->{'TxtSubmit'}),
		LSTITEM => $item
	 );

	# End form
	print $cgi->end_form(), "\n";
}


#####################
# Information Handler
#
# Will take the parameters and depending on information received can change the
# passwords and email.  The rest is considered non-essential.

sub InformationHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $config   = shift;					# Configuration hash
	
	my $audit;						# Auditing handle
	my $choice   = 0;					# Choice of action
	my $ret;						# Return value


	#####################
	# Program area	
	
	# Primary security check
	# Password Validity
	if (CheckDataPassword($database, $param->{'USER'}, $param->{'OLDPASS'}))
	{ 
		# Increment by one
		$choice++; 
	}
	else
	{ 
		# Display error message
		$ret = $config->{'ErrPassword'}; 
	}


	# Password change
	if (defined($param->{'ONEPASS'}))
	{
		# Valid new passwords
		if ((CheckDataLenght($param->{'ONEPASS'})) && ($param->{'ONEPASS'} eq $param->{'TWOPASS'}) )
		{
			# Inrement by one 
			$choice++; 
		}
		else
		{
			# Display error message 
			$ret = $config->{'ErrMismatch'}; 
		}
	}	

	# E-Mail change
	elsif (defined($param->{'EMAIL'}))
	{
		# Valid new E-Mail
		if (CheckDataEMail($param->{'EMAIL'}))
		{	
			# Inrement by one 
			$choice++; 
		}
		else
		{ 
			# Display error message
			$ret = $config->{'ErrEMail'}; 
		}
	}

	# If both tests pass, change data
	if ($choice == 2)
	{
		# Auditing handle
		$audit = new Ethereal::Audit($database, $cgi, $param);

		# Evaluate change
		eval
		{
			# Password change
			if ( defined($param->{'ONEPASS'}) )
			{
				# Create random string
				my $rand = md5_hex(rand());
				$rand = substr($rand, 0, 14);			

				# Insert new password
				$database->Write("UPDATE Puppeteer 
					SET   PuppeteerPassword=? 
					WHERE PuppeteerLogin=?", 

				 crypt(substr($param->{'ONEPASS'}, 0, 8), $rand), 
			 	 $param->{'USER'}
				);

				# Record
				$audit->PasswordChange($param->{'USER'});
			}
		
			# Email change
			elsif ( defined($param->{'EMAIL'}) )
			{
				# Insert new password
				$database->Write("UPDATE Puppeteer 
					SET PuppeteerEMail=? 
					WHERE PuppeteerLogin=?", 

				 $param->{'EMAIL'}, 
				 $param->{'USER'}
				);

				# Record
				$audit->EmailChange($param->{'USER'} . ':' . $param->{'EMAIL'});
			}


			# Commit changes
			$database->Commit();
		};

		# If block failed
		if ($@)
		{
			# Issue warning
			warn("Transaction aborted: $@");

			# Undo incomplete changes
			$database->Rollback();
		}


		# Notify of Success
		$ret = $config->{'ErrSuccess'};
	}

	# Return
	return $ret;
}


#####################
# InitialDisplay
#
# Handles the general output of the initial display screen.  Which will allow one
# to change password, E-Mail addresses and/or passwords.

sub InitialDisplay						# Database, CGI, Param
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $config   = shift;					# Configuration hash
	my $pass     = shift;					# Status display
	my $upload   = shift;					# Upload handle

	my $option;						# Option handler

	my $url      = $cgi->url(-absolute=>1);			# Self referencing URL
	my $links;						# List of links

	my $check;						# Quick check

	my $active;						# Active handles
	my $inactive;						# Inactive handles

	my @active;						# Active handles
	my @inactive;						# Inactive handles
	
	my %args;						# Arguments
	

	#####################
	# Program area	

	# Quick check on user
	($check) = $database->DataGetSocialLocate($param->{'USER'});


	# Argument handling
	# Define base arguments
	%args = (
		USER  => $param->{'USER'},
		CRYPT => $param->{'CRYPT'}
	 );

	# Generate Inlines
	# General settings
	$links  = $tmpl->Pass('TagTabbed',
	        MLINK => $url . $param->EmbedInline(%args),
		MNAME => $config->{'TagPref'}
	 );

	# Email address
	$links .= $tmpl->Pass('TagTabbed',
	        MLINK => $url . $param->EmbedInline(%args, CMAIL=>'True'),
		MNAME => $config->{'TagMail'}
	 );

	# Password Changes
	$links .= $tmpl->Pass('TagTabbed',
	        MLINK => $url . $param->EmbedInline(%args, PASSWORD=>'True'),
		MNAME => $config->{'TagPass'}
	 );

	# Contact information
	$links  .= $tmpl->Pass('TagTabbed',
		MLINK => $url . $param->EmbedInline(%args, CONTACT=>'True'),
		MNAME => $config->{'TagContact'}
	 );


	# Social aspect code
	# Social networking aspect
	$links .= $tmpl->Pass('TagTabbed',
	        MLINK => $url . $param->EmbedInline(%args, SOCIAL=>'True'),
		MNAME => $config->{'TagSocial'}
	 );
	 
	# Vanity pictures upload
	$links .= $tmpl->Pass('TagTabbed',
	        MLINK => $url . $param->EmbedInline(%args, VANITY=>'True'),
		MNAME => $config->{'TagVanity'}
	 ) if (defined($check));


	
	# Behavioral differences
	# Password change
	if (defined($param->{'PASSWORD'}))
	{
		# Display of information
		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), PASSWORD=>'True'), "\n";

		# Display password change template
		$tmpl->Show('TmplUserPass',
			MLINKS  => $links,
			MPASS   => $pass,
			WPOLD   => $cgi->password_field('OLDPASS', '', 10, 15),
			WPNEW   => $cgi->password_field('ONEPASS', '', 10, 15),
			WPSEC   => $cgi->password_field('TWOPASS', '', 10, 15),
			WSUBMIT => $cgi->submit($config->{'TxtChange'})
		 );

		# Embed
		print $cgi->end_form(), "\n";
	}

	# Email address change
	elsif (defined($param->{'CMAIL'}))
	{
		# Pull Email
		my ($email) = $database->DataGetPuppeteerEmail($param->{'USER'});


		# Display of information
		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), CMAIL=>'True');

		# Display email change template
		$tmpl->Show('TmplUserMail',
			MLINKS  => $links,
			MPASS   => $pass,
			WPOLD   => $cgi->password_field('OLDPASS', '', 10, 15),
			WEMAIL  => $cgi->textfield('EMAIL', $email, 30, 45),
			WSUBMIT => $cgi->submit($config->{'TxtChange'})
		 );

		# Embed
		print $cgi->end_form(), "\n";
	}

	# Contact information
	elsif (defined($param->{'CONTACT'}))
	{ 
		# Display and handle information
		ContactHandler($database, $cgi, $param, $tmpl, \%config, $links);
 	}


	# Social networking
	# Confirmation of locations
	elsif (defined($param->{'GEO'}))
	{

		# Social networking handler
		SocialConfirm($database, $cgi, $param, $tmpl, \%config, $links);
	}

	# Social networking profile
	elsif (defined($param->{'SOCIAL'}))
	{

		# Social networking handler
		SocialProfile($database, $cgi, $param, $tmpl, \%config, $links);
	}

	# Social networking vanity shots
	elsif ((defined($param->{'VANITY'}))
	 && (defined($check)))
	{
		# Vanity can be fairly complex
		VanityHandler($database, $cgi, $param, $tmpl, \%config, $links, $upload);
	}
	

	# Chat prefernces
	else
	{
		#
		# Additional Variables

		my $sjava;					# Formated javascript
		my $schat;					# Formatted chat preferences
		my $sformat;					# Formatted format options
		my $signore;					# Ignore control
		my $smailer;					# Mailer behaviour
		my $sprivacy;					# Privacy options

		my $lsthost;					# Hosting list
		my $rjava;					# Formated javascript
		my $rchat;					# Formatted chat preferences
		my $rformat;					# Formatted format options
		my $rignore;					# Ignore wigets
		my $rmailer;					# Mailer behaviour
		my $rprivacy;					# Privacy options

		my @rjava;					# Javascript options
		my @rchat;					# Chat options
		my @rformat;					# Formatting options
		my @rignore;					# Ignore options
		my @rmailer;					# Mailer behaviour options
		my @rprivacy;					# Privacy options


		# Retreive privs
		my ($super, $admin, $mod) = $database->DataGetPrivs($param->{'USER'});

		# Pull current options
		($sjava, $schat, $sformat, $signore, $smailer, $sprivacy) = $database->DataGet("SELECT
			  PuppeteerJavascript,
			  PuppeteerChatPref,
			  PuppeteerFormatting,
			  PuppeteerIgnore,
			  PuppeteerMailer,
			  PuppeteerPrivacy
			FROM Puppeteer
			WHERE PuppeteerLogin=?",

		 $param->{'USER'}
		);

		# Create new instance
		$option = new Ethereal::Option();

		# Split data into useable arrays for display arrays
		$option->Split(\@rjava, $config->{'OptJavascript'});
		$option->Split(\@rchat, $config->{'OptChatPref'});
		$option->Split(\@rformat, $config->{'OptFormat'});
		$option->Split(\@rmailer, $config->{'OptMailer'});
		$option->Split(\@rprivacy, $config->{'OptPrivacy'});


		# Data check
		$param->{'CJAVA'}    = $option->FindJavascriptRev($config->{'OptJavascript'}, $sjava);
		$param->{'CCHAT'}    = $option->FindChatPrefsRev($config->{'OptChatPref'}, $schat);
		$param->{'CFORMAT'}  = $option->FindFormattingRev($config->{'OptFormat'}, $sformat);
		$param->{'CMAILER'}  = $option->FindMailerRev($config->{'OptMailer'}, $smailer);
		$param->{'CPRIVACY'} = $option->FindPrivacyRev($config->{'OptPrivacy'}, $sprivacy);

		# Format options nicely
		$rjava    = $cgi->radio_group('CJAVA',    \@rjava,    $param->{'CJAVA'},    'true');
		$rchat    = $cgi->radio_group('CCHAT',    \@rchat,    $param->{'CCHAT'},    'true');
		$rformat  = $cgi->radio_group('CFORMAT',  \@rformat,  $param->{'CFORMAT'},  'true');
		$rmailer  = $cgi->radio_group('CMAILER',  \@rmailer,  $param->{'CMAILER'},  'true');
		$rprivacy = $cgi->radio_group('CPRIVACY', \@rprivacy, $param->{'CPRIVACY'}, 'true');

		# Supervisory feature
		# Initialize
		$lsthost = '';


		# Format options nicely
		if (($super eq 'yes') 
		 || ($mod eq 'yes'))
		{
			# Split up	
			$option->Split(\@rignore, $config->{'OptScope'});
	
			# Data check		
			$param->{'CIGNORE'}  = $option->FindScopeRev($config->{'OptScope'}, $signore);

			# Ignore wiget
			$rignore = $cgi->radio_group('CIGNORE',  \@rignore,  $param->{'CIGNORE'}, 'true');

			# Configure template
			$lsthost = $tmpl->Pass('TmplUserPrefHost',
				WIGNORE => $rignore
			 );
		}	


		# Initial display
		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), SETPREFS=>'TRUE');

		# Display preferences template
		$tmpl->Show('TmplUserPref',
			MLINKS   => $links,
			LSTHOST  => $lsthost,
			MPASS    => $pass,
			WPRIVACY => $rprivacy,
			WJAVA    => $rjava,
			WCHAT    => $rchat,
			WFORMAT  => $rformat,
			WMAILER  => $rmailer,				
			WSUBMIT  => $cgi->submit($config->{'TxtChange'})
		 );

		# Embed
		print $cgi->end_form(), "\n";
	}
}

#####################
# Social Profile
#
# The social profile for a user is handled for the most part through this
# interface with handles all aspects of the handling.

sub SocialProfile
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $config   = shift;					# Configuration hash
	my $links    = shift;					# Hash of links

	my $filter;						# Filter handler
	my $option;						# Options handler

	my $statement;						# Database statement
	my $res;						# Database reference

	my $url      = $cgi->url(-absolute=>1);			# Self referencing link
	my $igeo;						# Embeded link

	my $compare;						# Compare value
	my $current;						# Current key
	my $lookup;						# Lookup clause
	my $ref;						# Referenced field
	my $style;						# Widget style
	my $style_min;						# Widget style min size
	my $style_max;						# Widget style max size

	my $loop;						# Looping value
	my $def;						# Defaulted value
	my $val;						# Actual value
	my $wig;						# Wigit creation

	my $tpl;						# Template run through
	my $upd      = '';					# Update query information
	my $dte      = '';					# Date of birth
	my $mes      = '';					# Message updates

	my @elem;						# Available elements
	my @pass;						# Passage of information
		

	# Create complex structure with all requirements
	# That way we can loop and get what is required

	# 0-4     - Lookup for validity
	# A-Za-z  - Field in database / Tags
	# A-za-z  - Type of widget to generate Combo/Radio/Area/Text
	# 0-9     - Smallest entry of field
	# 0-9     - Largest entry of field

	my %selm = (
		SPGENDER   => [1, 'PhysicalGender',     'combo', 0,  0],
		SPBDAY     => [1, 'PhysicalBirthDay',   'combo', 0,  0],
		SPBMONTH   => [2, 'PhysicalBirthMonth', 'combo', 0,  0],
		SPBYEAR    => [3, 'PhysicalBirthYear',  'text',  4,  4],
		SPHEIGHT   => [1, 'PhysicalHeight',     'combo', 0,  0],
		SPBODY     => [1, 'PhysicalBodyType',   'combo', 0,  0],
		SPEYES     => [1, 'PhysicalEyeColour',  'combo', 0,  0],
		SPHAIR     => [1, 'PhysicalHairColour', 'combo', 0,  0],
		SPETHNIC   => [1, 'PhysicalEthnicity',  'combo', 0,  0],
		SIPHRASE   => [0, 'IntroCatchPhrase',   'text',  65, 125],
		SIDESCR    => [0, 'IntroDescription',   'area',  75, 6],
		SIDATE     => [0, 'IntroFirstDate',     'area',  75, 6],
		SSLOOKING  => [1, 'SocialLooking',      'combo', 0,  0],
		SSSEEKING  => [1, 'SocialSeeking',      'combo', 0,  0],
		SSAGEMIN   => [4, 'SocialAgeMin',       'text',  2,  2],
		SSAGEMAX   => [4, 'SocialAgeMax',       'text',  2,  2],
		SSMARITAL  => [1, 'SocialMarital',      'combo', 0,  0],
		SSHCHILD   => [1, 'SocialChildrenHave', 'combo', 0,  0],
		SSWCHILD   => [1, 'SocialChildrenWant', 'combo', 0,  0],
		SSINTEREST => [0, 'SocialInterests',    'text',  75, 1024],
		SSRELIGION => [1, 'SocialReligion',     'combo', 0,  0],
		SSWORK     => [0, 'SocialProfession',   'text',  30, 30],
		SVSMOKE    => [1, 'ViceSmoke',          'radio', 0,  0],
		SVDRINK    => [1, 'ViceDrink',          'radio', 0,  0]
	);

	#
	# Comment necessary to prevent accidental meldown of core tags
	# removal when checking for unsued aspects of the code.
	#
	# OptPhysicalGender
	# OptPhysicalBirthDay
	# OptPhysicalBirthMonth
	# OptPhysicalBirthYear
	# OptPhysicalHeight
	# OptPhysicalBodyType
	# OptPhysicalEyeColour
	# OptPhysicalHairColour
	# OptPhysicalEthnicity
	# OptIntroCatchPhrase
	# OptIntroDescription
	# OptIntroFirstDate
	# OptSocialLooking
	# OptSocialSeeking
	# OptSocialMarital
	# OptSocialChildrenHave
	# OptSocialChildrenWant
	# OptSocialInterests
	# OptSocialReligion
	# OptSocialProfession
	# OptViceSmoke
	# OptViceDrink

	# Create new instance
	$filter = new Ethereal::Filter();
	$option = new Ethereal::Option();


	# Basics in place
	# Draw all necessary information
	$database->Pull(\$statement, "SELECT
		  PuppeteerLogin       AS \"PuppeteerLogin\",
		  PuppeteerTimestamp   AS \"PuppeteerTimestamp\",
		  PuppetName           AS \"PuppetName\",
		  LocateCountry        AS \"LocateCountry\",
		  PhysicalGender       AS \"PhysicalGender\",
		  PhysicalBirthDay     AS \"PhysicalBirthDay\",
		  PhysicalBirthMonth   AS \"PhysicalBirthMonth\",
		  PhysicalBirthYear    AS \"PhysicalBirthYear\",
		  PhysicalHeight       AS \"PhysicalHeight\",
		  PhysicalBodyType     AS \"PhysicalBodyType\",
		  PhysicalEyeColour    AS \"PhysicalEyeColour\",
		  PhysicalHairColour   AS \"PhysicalHairColour\",
		  PhysicalEthnicity    AS \"PhysicalEthnicity\",
		  IntroCatchPhrase     AS \"IntroCatchPhrase\",
		  IntroDescription     AS \"IntroDescription\",
		  IntroFirstDate       AS \"IntroFirstDate\",
		  SocialLooking        AS \"SocialLooking\",
		  SocialSeeking        AS \"SocialSeeking\",
		  SocialAgeMin         AS \"SocialAgeMin\",
		  SocialAgeMax         AS \"SocialAgeMax\",
		  SocialMarital        AS \"SocialMarital\",
		  SocialChildrenHave   AS \"SocialChildrenHave\",
		  SocialChildrenWant   AS \"SocialChildrenWant\",
		  SocialInterests      AS \"SocialInterests\",
		  SocialReligion       AS \"SocialReligion\",
		  SocialProfession     AS \"SocialProfession\",
		  ViceSmoke            AS \"ViceSmoke\",
		  ViceDrink            AS \"ViceDrink\"
		 FROM Network
		WHERE PuppeteerLogin=?",
			
	 $param->{'USER'}
 	);	


	#
	# User has an entry already
	if (($res = $statement->fetchrow_hashref())
	 && (defined($res->{'LocateCountry'})))
	{
		# Template
		$tpl = $config->{'TmplSocial'};
		

		# For each element in look
		# Check data integrity, compare and create wiget
		foreach $current (sort(keys(%selm)))
		{
			# Make sure
			$val = undef;
			
			# Clarify settings
			($lookup, $ref, $style, $style_min, $style_max) = @{$selm{$current}}; 

			# Behavioural based handling of values
			# Lookup required
			if ($lookup == 1)
			{
				# Split the field
				$option->Split(\@elem, $config->{'Opt' . $ref});


				# Comparisons
				# Ensure integrity
				if ($param->{'SOCIAL'} eq 'update')
				{
					# What to compare with
					$compare = (defined($param->{$current}))
					  ? $param->{$current}
					  : $res->{$ref};


					# Simple look to determin actions
					foreach $loop (@elem)
					{
						# Assing accordingly
						$val = $loop;
						
						# Match found, move on
						last if ($compare eq $val);
					}

					# Pop for good measure
					pop(@elem);
				}

				# Start fresh
				else
				{
					# Default always last entry
					$def = pop(@elem);

					# Default wins
					$val = (defined($res->{$ref}))
				         ? $res->{$ref}
					 : $def;
				}
			}

			# Month handler
			elsif ($lookup == 2)
			{
				# Split the field
				$option->Split(\@elem, $config->{'Opt' . $ref});


				# Handling method
				# Database operation
				unless ($param->{'SOCIAL'} eq 'update')
				{
					# Set value
					$val = (defined($res->{$ref}))
					  ? $elem[($res->{$ref})-1]
					  : $elem[0];
				}

				# Parameter based
				else
				{
					# Simple look to determin actions
					foreach $loop (@elem)
					{
						# Assing accordingly
						$val = $loop;
						
						# Match found, move on
						last if ($param->{$current} eq $val);
					}
				}

				# Push off for good luck
				pop(@elem);
			}

			# Text goodness
			else
			{
				# Provides reasonable starting points
				$val = (defined($param->{$current}))
				  ? $param->{$current}
				  : $res->{$ref};

				# Double check
				# Filter accordingly
				$val = (defined($val))
				  ? $filter->SafeHTML($val)
				  : '';


				# Work from here
				# Year restriction
				$val = $config->{'SetYear'}
				  if (($lookup == 3) && ($val !~ /^\d{4,4}$/));

				# Size check
				$val = ''
				  if (($lookup < 3) && (length($val) < 5)) ;
			}

			#
			# Create update code
			# Generate what is necessary
			if ($param->{'SOCIAL'} eq 'update')
			{
				# Not related to birth date
				if ($ref !~ /^PhysicalBirth/)
				{
					# Manipulate string
					$upd .= " $ref=\?,";

					# Push onto array
					push(@pass, $val);
				}

				else
				{
					# Append to string formatted date
					$dte .= "$val-";
				}
			}


			#
			# Wigit generation
			# Change behaviour last on it
			if ($style eq 'combo')
			{
				# Combo box/popup menu
				$wig = $cgi->popup_menu(
					-name      => $current,
					-values    => \@elem,
					-default   => $val
				 );
			}

			# Radio buttons
			elsif ($style eq 'radio')
			{
				# Radio group
				$wig = $cgi->radio_group(
					-name      => $current,
					-values    => \@elem,
					-default   => $val,
					-linebreak =>'true'
				 );
			}

			# Textarea
			elsif ($style eq 'area')
			{
				# Min and Max are used for Rows and Columns respectively
				$wig = $cgi->textarea(
					-name      => $current,
					-value     => $val,
					-rows      => $style_max,
					-columns   => $style_min
				 );
			}
			
			# Text field
			elsif ($style eq 'text')
			{
				# Min and Max are used as their name implies
				$wig = $cgi->textfield(
					-name      => $current,
					-value     => $val,
					-size      => $style_min,
					-maxlength => $style_max
				 );
			}

			# Template handling
			# Adapt slightly
			$current = 'W' . $current;
			
			# Search and replace
			$tpl =~ s/$current/$wig/gs;
		}

		# Quickly update
		# If necessary
		if ($param->{'SOCIAL'} eq 'update')
		{
			# Cleanup
			chop($dte);

			# Evaluate what needs to be done
			eval
			{
				# Nice and simple
				$database->Write("UPDATE Social
					  SET $upd PhysicalBirthDate=?
				       	WHERE PuppeteerLogin=?",
				
				 @pass,
			 	 $dte,
			 	 $param->{'USER'}
			 	);

				# Commit changes
				$database->Commit();

				# Message handling
				$mes = $config->{'MsgSocialUpdate'};
			};

			# Evil errors
			if ($@)
			{
				# Message handling
				$mes = $config->{'MsgSocialError'};

				# Rollback changes
				$database->Rollback();
			}
		}

		# Allow users to re-confirm location
		$igeo = $param->EmbedInline($param->Flat(),
			GEO    => 'initial'
		 );

		# Generate widget
		$wig =  $cgi->submit($config->{'TxtChange'});


		# Quick replace
		$tpl =~ s/LLOCATE/$url$igeo/gs;
		$tpl =~ s/MLINKS/$links/gs;
		$tpl =~ s/MMESS/$mes/gs;
		$tpl =~ s/WSUBMIT/$wig/gs;


		# HTML display
		# Form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), SOCIAL=>'update');

		# Display preferences template
		print $tpl, "\n";

		# Embed
		print $cgi->end_form(), "\n";
	}

	#
	# User needs to be located prior
	else
	{
		# Get more details
		SocialConfirm($database, $cgi, $param, $tmpl, $config, $links);
	}

	# Finish query
	$statement->finish();
}


#####################
# Social Confirm
#
# This script is far simpler in application then the profile since it only deals
# with a finite, free flowing information gotten from various sources.

sub SocialConfirm
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $config   = shift;					# Configuration hash
	my $links    = shift;					# Hash of links

	my $geo;						# Geographical handle

	my $url      = $cgi->url(-absolute=>1);			# Self referencing link
	my $igeo;						# Embeded link
	
	my $mes      = '';					# Passes message

	my $addr;						# Something mortals can understand
	my $city;						# City location
	my $region;						# Region of inhabitation
	my $country;						# Queen and country
	my $lat;						# Latitude
	my $lon;						# Longitude
	

	#####################
	# Program Area

	# Writing operations
	if ((defined($param->{'GEO'}))
	 && ($param->{'GEO'} eq 'confirm'))
	{
		# Quick check
		($country) = $database->DataGetSocialLocate($param->{'USER'});
		
		# Massage information
		$param->{'GEOCIT'} = (length($param->{'GEOCIT'}) > 2) ? $param->{'GEOCIT'} : undef;
		$param->{'GEOCNT'} = (length($param->{'GEOCNT'}) > 2) ? $param->{'GEOCNT'} : undef;
		$param->{'GEOREG'} = (length($param->{'GEOREG'}) > 2) ? $param->{'GEOREG'} : undef;

		# Evaluate matters
		eval
		{
			# This is an update
			if (defined($country))
			{
				# Update entries
				$database->Write("UPDATE Social
					SET LocateCountry=?,
					    LocateRegion=?,
					    LocateCity=?,
					    LocateLat=?,
					    LocateLong=?
					WHERE PuppeteerLogin=?",

				 $param->{'GEOCNT'},
				 $param->{'GEOREG'},
				 $param->{'GEOCIT'},
				 $param->{'GEOLAT'},
				 $param->{'GEOLON'},
				 $param->{'USER'}
				);
			}

			# Creation
			else
			{
				# Update entries
				$database->Write("INSERT INTO Social
					(PuppeteerLogin,
					 LocateCountry,
					 LocateRegion,
					 LocateCity,
					 LocateLat,
					 LocateLong)
					VALUES (?,?,?,?,?,?)",

				 $param->{'USER'},
				 $param->{'GEOCNT'},
				 $param->{'GEOREG'},
				 $param->{'GEOCIT'},
				 $param->{'GEOLAT'},
				 $param->{'GEOLON'}
				);

			}

			# Commit changes
			$database->Commit();
		};

		# Bad user
		if ($@)
		{
			# Something smells rotten in the state of Denmark
			$mes = $config->{'MsgSocialError'};

			# Rollback
			$database->Rollback();
		}
	}
	

	# User details	
	# Check against the database first
	($country,
	 $region,
	 $city,
	 $lat,
	 $lon) = $database->DataGetSocialLocate($param->{'USER'});

	# Nothing in database
	unless (defined($lat))
	{
		# Debugging purposes
		$addr = (defined($config->{'SetAddressDebug'}))
		  ? $config->{'SetAddressDebug'}
		  : undef;
		
		# Resort to using geo targetting information
		# Create new instance
		$geo = new Ethereal::Geo($database);

		# Pull the information and try again
		($country,
		 $region,
		 $city,
		 $lat,
		 $lon) = $geo->Locate($addr);

		# Convert
		$lat = sprintf('%.3f', $lat);
		$lon = sprintf('%.3f', $lon);
		
	 	# Double check
		unless (defined($lat))
		{
			# Replace complately
			$lat = $config->{'SetLocateLat'};
			$lon = $config->{'SetLocateLong'};

			# Reset all
			$country = '';
			$region  = '';
			$city    = '';
		}

		else
		{
			# Check all elements
			($country,
			 $region,
			 $city) = NameCase($country, $region, $city);
		}
	}

	# Generate Google Maps link
	$igeo = $param->EmbedInline($param->Flat(),
		USER   => $param->{'USER'},
		CRYPT  => $param->{'CRYPT'},
		GEO    => 'geo', 
		GEOCIT => $city,
		GEOREG => $region,
		GEOCNT => $country,
		GEOLAT => $lat,
		GEOLON => $lon
	 );


	# HTML display
	# Form
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), GEO=>'confirm', GEOLAT=>$lat, GEOLON=>$lon);

	# Display template
	$tmpl->Show('TmplSocialConfirm',
		MLINKS   => $links,
		MMESSAGE => $mes,
		MLAT     => $lat,
		MLON     => $lon,
		LGEO     => $url . $igeo,
		WCITY    => $cgi->textfield('GEOCIT', $city, 30, 45),
		WREGION  => $cgi->textfield('GEOREG', $region, 30, 45),
		WCOUNTRY => $cgi->textfield('GEOCNT', $country, 30, 45),
		WSUBMIT  => $cgi->submit($config->{'TxtChange'})
	 );

	# Embed
	print $cgi->end_form(), "\n";
}


#####################
# Social Locate
#
# This is the prime interface to work directly with Google Maps in order to
# find the users proper location.

sub SocialLocate
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handler
	my $config   = shift;					# Configuration hash

	my $url      = $cgi->url(-absolute=>1);			# Self referencing link
	my $igeo;						# Embeded link
	my $links;						# Hash of links

	my $efrm;						# End of form
	my $sfrm;						# Start of form

	my %system;						# System hash


	#####################
	# Program Area

	# Generate Google Maps link
	$igeo = $param->EmbedInline($param->Flat(),
		GEO    => 'initial', 
	 );

	# Generatr links
	$links = $tmpl->Pass('TagTabbed',
		MNAME => $config->{'TxtBack'},
		MLINK => $url . $igeo
	 );


	# Link with has
	$database->GetHashSystem(\%system);
	
	
	# Prepare
	# Start form
	$sfrm = $cgi->start_form(-name=>'LATLON')
	      . $param->EmbedNormal($param->Flat(), GEO=>'confirm');

	# End form	
	$efrm = $cgi->end_form() . "\n";


	# Display template
	$tmpl->Show('TmplSocialLocate',
		MLINKS   => $links,
		MGMKEY   => $system{'SetGoogleMaps'},
		MSLAT    => $param->{'GEOLAT'},
		MSLON    => $param->{'GEOLON'},
		WSTRFORM => $sfrm,
		WENDFORM => $efrm
	 );
}


#####################
# Vanity Handler
#
# The displaying of vanity pictures to outsiders is handled through this
# specific interface.

sub VanityHandler
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handle
	my $config   = shift;					# Configuration hash
	my $links    = shift;					# Hash of links
	my $upload   = shift;					# Upload template

	my $page;						# Homepage handler
	my $image;						# Image handling
	
	my $res;						# Search results
	my $statement;						# Search statements

	my $count;						# Number of images

	my $default;						# Default image
	my $ocount;						# Outside counter	
	
	my $defs;						# Number of defaults
	my $height;						# Height measurement
	my $width;						# Width measurement

	my $data;						# Image contents
	my $ext;						# File name
	my $mime;						# Mime types
	my $path;						# Path updates
	my $short;						# Short name

	my $def;						# Default identifier
	my $id;							# Identification number
	my $pub;						# Public identifier
	my $vote;						# Vote identifier

	my $score;						# Score
	my $votes;						# Total votes
	
	my $current;						# Current items on display
	my $listing;						# Listing of all vanity shots
	my $uploads;						# Creation template

	my $wdelete;						# Deletion widget
	my $wpublic;						# Public entity widget
	my $wdefault;						# Default widget
	my $wvote;						# Coting widget

	my @img;						# Image number selection


	#####################
	# Program area	

	# Stress bearing directory
	$short = substr(lc($param->{'USER'}), 0, 3);
	$path  = $config->{'SetVanityPath'} . '/' . lc($short);

	# Create instance
	$page  = new Ethereal::Home($database, $path);


	# Create a new shot
	# Check for requirement
	if (defined($upload))
	{
		# Cannot go over the limit
		($count) = $database->DataGet("SELECT COUNT(*)
				FROM Vanity
				WHERE PuppeteerLogin=?",

		 $param->{'USER'}
		);

		# Safety
		$count = (defined($count)) ? $count : 0;


		# Within tolerence levels
		if ($count < $config->{'SetVanityLimit'})
		{
			# Basics
			$mime =  $cgi->uploadInfo($param->{'UPLOAD_VANITY'})->{'Content-Type'};
			$ext  =  $param->{'UPLOAD_VANITY'};

			# Extension
			$ext  =~ s/^(.*\/|^)(.*?)(\.)(\w+)$/$4/;


			# Check mime types
			# Acceptable mime type
			if ($mime =~ /^image/i)
			{
				# Create instances
				$image = new Ethereal::Image($database, $path);


				# Check for directory existence
				$page->DirCreate($path) unless (-d $path);

				# Final check and proceed
				if (-d $path)
				{
					# Pull ID
					($id) = $database->DataGetInsert('seqMail');
					

					# Copy contents
					while (<$upload>)
					{
						# Append until complete
						$data .= $_;
					}


					# Testing again
					eval
					{
						# Create full and thumbnail
						$page->FileWrite("$id.$ext", $data);
						$page->FileWrite("$id-thumb.$ext", $data);

						# Pull sizes to check
						($width, $height) = $image->Size("$id.$ext");


						# Lets skip if too small
						if (($width  > $config->{'SetVanityThumb'})
						 && ($height > $config->{'SetVanityThumb'}))
						{
							# Resizing
							# Fill sized image
							$image->Set("$id.$ext", $config->{'SetVanityMax'})
							  unless (($width  < $config->{'SetVanityMax'})
							       && ($height < $config->{'SetVanityMax'}));

						       	# Thumbnail
							$image->Set("$id-thumb.$ext", $config->{'SetVanityThumb'});

							# Determine defaults
							$def = ($count != 0) ? 'no' : 'yes';

						
							# Pull new sizes sizes
							($width, $height) = $image->Size("$id.$ext");

							# Database update
							$database->Write("INSERT INTO Vanity
								(VanityID,
								 PuppeteerLogin,
								 VanityPath,
								 VanityThumb,
								 VanityWidth,
								 VanityHeight,
								 VanityDefault)
								VALUES (?,?,?,?,?,?,?)",

							  $id,
							  $param->{'USER'},
							  $config->{'LnkVanity'} . "\/$short\/$id.$ext",
							  $config->{'LnkVanity'} . "\/$short\/$id-thumb.$ext",
							  $width,
							  $height,
							  $def
							 );


						}

						# Too small
						else
						{
							# Remove and forget about it
							$page->Unlink("$id.$ext", $data);
							$page->Unlink("$id-thumb.$ext", $data);
						}

						# Commit
						$database->Commit();
					};

					# Has there been an error
					if ($@)
					{
						# Warn
						warn('There was a transaction error!', $@);

						# Rollback
						$database->Rollback();
					}
				}
			}
		}
	}

	
	# Active shots
	# Initial user search and gather
	$database->Pull(\$statement, "SELECT
		  VanityID      AS \"VanityID\",
		  VanityPath    AS \"VanityPath\",
		  VanityThumb   AS \"VanityThumb\",
		  VanityPublic  AS \"VanityPublic\",
		  VanityDefault AS \"VanityDefault\",
		  VanityVote    AS \"VanityVote\"
		 FROM Vanity
		WHERE PuppeteerLogin=?
		ORDER BY VanityID",

	 $param->{'USER'}
	);

	# Main check
	if ($res = $statement->fetchrow_hashref())
	{
		# Initialize
		$listing = '';

		# Counter
		$count   = 0;
		$ocount  = 0;
		
		# All or nothing
		eval
		{
			# Loop and gather
			do
			{
				# Always increment
				$ocount++;
				
				# Assign for ease
				$id = $res->{'VanityID'};

				# Check for removal
				if (defined($param->{"VDEL_$id"}))
				{
					# Remove and do not display
					$database->Write("DELETE FROM Vanity
						WHERE VanityID=?
						  AND PuppeteerLogin=?",
	
					  $id,
					  $param->{'USER'}
					 );

					# Find extension
					$ext  =~ s/^(.*\/|^)(.*?)(\.)(\w+)$/$4/;

					# Remove files
					$page->FileUnlink("$id\.$ext");
					$page->FileUnlink("$id-thumb\.$ext");
				}

			
				# Display as normal
				else
				{
					# Increment
					$count++;

					# Application of logic
					if ((defined($param->{'VANITY_PROCESS'}))
					 && (!defined($param->{$config->{'TxtUpload'}})))
					{
						# Defaults
						# This is a default
						if ($param->{'VDEF'} == $count)
						{
							# Set default settings
							$def     = 'yes';

							# Record it for next pass
							$default = $count;
						}

						# Regular image
						else
						{
							# Set to the default
							$def = 'no';
						}


						# Verification of matches
						# Public and vote
						$pub  = (defined($param->{"VPUB_$id"})) ? 'yes' : 'no';
						$vote = (defined($param->{"VVTE_$id"})) ? 'yes' : 'no';

						# Are changes necessary
						if (($def  ne $res->{'VanityDefault'})
						 || ($pub  ne $res->{'VanityPublic'})
						 || ($vote ne $res->{'VanityVote'}))
					 	{
							# Update entry
							$database->Write("UPDATE Vanity
								SET VanityDefault=?,
								    VanityPublic=?,
								    VanityVote=?
								WHERE VanityID=?
								  AND PuppeteerLogin=?",

							 $def,
							 $pub,
							 $vote,
							 $id,
							 $param->{'USER'}
							 );
						}
					}
	
	
					# Dump defaults
					else
					{
						# Assign database defaults
						$pub  = $res->{'VanityPublic'};
						$vote = $res->{'VanityVote'};


						# Find default
						$default = $count if ($res->{'VanityDefault'} eq 'yes');
					}

					# Fetch scores
					($score, $votes) = $database->DataGetVanityScores($id);

					# Ensure definition
					$score = (defined($score)) ? int($score) : 0;
					
			
					# Generate widgets
					# Removal
					$wdelete = $cgi->checkbox(-name=>"VDEL_$id", -label=>'');

					# Public
					$wpublic = ($pub eq 'yes')
					  ? $cgi->checkbox(-name=>"VPUB_$id", -label=>'', -checked=>'yes')
					  : $cgi->checkbox(-name=>"VPUB_$id", -label=>'');
			
					# Vote
					$wvote = ($vote eq 'yes')
					  ? $cgi->checkbox( -name=>"VVTE_$id", -label=>'',  -checked=>'yes')
					  : $cgi->checkbox( -name=>"VVTE_$id", -label=>'');


					# Append to template
					$listing .= $tmpl->Pass('TmplVanityListing',
						LTHUMB   => $res->{'VanityThumb'},
						MID      => $id,
						MNUM     => $count,
						MSCORE   => $score,
						MVOTES   => $votes,
						WDELETE  => $wdelete,
						WPUBLIC  => $wpublic,
						WVOTE    => $wvote
					 );
				}

			} while ($res = $statement->fetchrow_hashref());


			# Ensure defaults are correct
			# Since a trigger may cause loops
			($defs) = $database->DataGet("SELECT COUNT(*) FROM Vanity
				WHERE PuppeteerLogin=?
				  AND VanityDefault=?",
				  
			 $param->{'USER'},
			 'yes'
		 	);

			# Work it out
			if ($defs != 1)
			{
				# Fetch an identifier
				($id) = $database->DataGet("SELECT VanityID
					 FROM Vanity
					WHERE PuppeteerLogin=?
					ORDER BY VanityID",
					
				  $param->{'USER'}
				);

				# There is a bite
				if (defined($id))
				{
					# Reset all
					$database->Write("UPDATE Vanity
						  SET VanityDefault=?
						WHERE PuppeteerLogin=?",

					 'no',
					 $param->{'USER'}
				 	);

					# Set one as default
					$database->Write("UPDATE Vanity
						  SET VanityDefault=?
						WHERE VanityID=?",

					 'yes',
					 $id
				 	);
				}	
			}


			# Commit changes
			$database->Commit();
		};


		# Something has failed
		if ($@)
		{
			# Warn
			warn('There was a transaction error!', $@);

			# Rollback
			$database->Rollback();
		}

		
		# Generate widget
		# Populate array
		for ($ocount=1; $ocount <= $count; $ocount++)
		{
			# Push in those numbers
			push(@img, $ocount);
		}

		# Create widget
		$wdefault = $cgi->radio_group(
			-name    => 'VDEF',
                        -values  => \@img,
                        -default => $default
		 );
	}

	# End query
	$statement->finish();


	# Finalize templates
	# Determine current listing
	$current = ($count > 0)
	  ? $tmpl->Pass('TmplVanityCurrent',
		  	LSTCURRENT => $listing,
			WDEFAULT   => $wdefault,
			WUPDATE    => $cgi->submit($config->{'TxtSubmit'}))
	  : '';

	# Uploading capability
	$uploads = ($count <= $config->{'SetVanityLimit'})
	  ? $tmpl->Pass('TmplVanityUpload',
		  	WUPLOAD => $cgi->filefield('UPLOAD_VANITY', '', 40, 140),
			WSUBMIT => $cgi->submit($config->{'TxtUpload'}))
	  : '';
	

	# Information display
	# Form
	print $cgi->start_multipart_form();
	print $param->EmbedNormal($param->Flat(), VANITY_PROCESS=>'True'), "\n";

	# Display template
	$tmpl->Show('TmplVanity',
		MLINKS   => $links,
		MCURRENT => $current,
		WUPLOAD  => $uploads
	 );

	# End form
	print $cgi->end_form(), "\n";
}

